import { takeLeading, call, put, select } from "redux-saga/effects";
import { ACTIONS } from "../actions";
import AuthAPI from "../apis/auth";
import {
  APP_STATE,
  ControlsType,
  KeystrokeResultType,
  Reducers,
  SignInPayload,
  UpdatePasswordPayload,
  USER_TYPE,
} from "../types";

function* signIn({
  payload,
  success,
}: {
  type: typeof ACTIONS.SIGN_IN;
  payload: SignInPayload;
  success: (state: string) => void;
}) {
  try {
    const data: { user: any } = yield call(AuthAPI.signin, payload);
    yield put({ type: ACTIONS.SET_USER_TYPE, payload: payload.userType });
    const auth = {
      ...data,
      organization:
        payload.userType === USER_TYPE.ORGANIZATION ? data.user : {},
      candidate: payload.userType === USER_TYPE.CANDIDATE ? data.user : {},
    };
    yield put({ type: ACTIONS.SET_AUTH, payload: auth });
    success(data.user.state);
  } catch (error) {
    success("failed");
  }
}

function* updateKeystrokeSettings({
  payload,
}: {
  type: typeof ACTIONS.UPDATE_KEYSTROKE_SETTINGS;
  payload: ControlsType;
}) {
  try {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.LOADING },
    });
    yield call(AuthAPI.updateControls, payload);
    const keystrokeResult: KeystrokeResultType = yield select(
      (state: Reducers) => state.auth.keystrokeResult
    );
    yield put({
      type: ACTIONS.SET_AUTH,
      payload: { keystrokeResult: { ...keystrokeResult, controls: payload } },
    });
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.SUCCESS },
    });
  } catch (error) {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.FAILED },
    });
  }
}

function* changePassword({
  payload,
}: {
  type: typeof ACTIONS.CHANGE_PASSWORD;
  payload: UpdatePasswordPayload;
}) {
  try {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.LOADING },
    });
    yield call(AuthAPI.updatePassword, payload);
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.SUCCESS },
    });
  } catch (error) {
    yield put({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.FAILED },
    });
  }
}

export default function* authSaga() {
  yield takeLeading(ACTIONS.SIGN_IN, signIn);
  yield takeLeading(ACTIONS.UPDATE_KEYSTROKE_SETTINGS, updateKeystrokeSettings);
  yield takeLeading(ACTIONS.CHANGE_PASSWORD, changePassword);
}
