import React, { useState, useEffect, ChangeEvent, useRef } from "react";
import { useDispatch, useSelector } from "react-redux";
import { ACTIONS } from "../common/actions";
import { updateOrganization } from "../common/actions/user"; 
import { getProfile } from "../common/lib/util";
import { APP_STATE, OrganizationType, USER_TYPE  } from "../common/types";
import Avatar from "./Avatar"; 
import ProfilePicCrop from "./Modals/ProfilePicCrop";

const OrganizationProfile = () => {
  const dispath = useDispatch();
  const org = useSelector(getProfile) as OrganizationType;
  const [profile, setProfile] = useState<OrganizationType>(org);
  const [profilePicPrev, setprofilePicPrev] = useState<string | null>(null); 
  const _profilePicInput = useRef<HTMLInputElement>(null);

  useEffect(() => {
    if (org) {
      setProfile(org); 
    }
  }, [org]);

  const onChange = (e: ChangeEvent<HTMLInputElement>) => {
    if (e.target.name === "phone") {
      setProfile({
        ...profile,
        contacts: { ...profile.contacts, phone: e.target.value },
      });
    } else {
      setProfile({
        ...profile,
        [e.target.name]: e.target.value,
      });
    }
  };

  const onChangeAddress = (e: ChangeEvent<HTMLInputElement>) => {
    setProfile({
      ...profile,
      contacts: {
        ...profile.contacts,
        address: {
          ...profile.contacts.address,
          [e.target.name]: e.target.value,
        },
      },
    });
  };

  const onUpdateOrg = (
    payload: Partial<OrganizationType>, 
  ) => {
    dispath(updateOrganization(payload));
  };

 
  const onSubmit = async () => {
    const {  _id, ...rest } = profile;
    dispath({
      type: ACTIONS.SET_APP_STATE,
      payload: { state: APP_STATE.LOADING },
    });
    onUpdateOrg(rest);
  };

  const onSelectProfilePic = (e: ChangeEvent<HTMLInputElement>) => {
    if (e.target.files) {
      setprofilePicPrev(URL.createObjectURL(e.target.files[0]));
    }
  };

  const onPressUploadProfilePic = () => {
    _profilePicInput?.current?.click();
  };

  const onRequestCloseProfilePic = () => {
    setprofilePicPrev(null);
  };

  return (
    <div className="card p-4 mb-3">
      <h5>Edit/Update profile</h5>
      <div className="mb-3 row">
        <label className="col-sm-3 col-form-label">Org Name</label>
        <div className="col-sm-9">
          <input
            type="text"
            className="form-control"
            name="name"
            value={profile.name}
            onChange={onChange}
          />
        </div>
      </div>

      <div className="mb-3 row">
        <label className="col-sm-3 col-form-label">Address</label>
        <div className="col-sm-9">
          <input
            type="text"
            name="addressLine"
            className="form-control mb-2"
            placeholder="Address line"
            value={profile.contacts?.address?.addressLine}
            onChange={onChangeAddress}
          />
          <input
            type="text"
            className="form-control mb-2"
            placeholder="City"
            name="city"
            value={profile.contacts?.address?.city}
            onChange={onChangeAddress}
          />
          <input
            type="text"
            className="form-control mb-2"
            placeholder="Country"
            name="country"
            value={profile.contacts?.address?.country}
            onChange={onChangeAddress}
          />
        </div>
      </div>
      <div className="mb-3 row">
        
        <div className="col-sm-6">
          <div className="row">
            <label className="col-sm-3 col-form-label">Phone</label>
            <div className="col-sm-9">
              <input
                type="text"
                className="form-control"
                name="phone"
                value={profile.contacts?.phone}
                onChange={onChange}
              />
            </div>
          </div>
        </div>
        <div className="col-sm-6">
          <div className="row">
            <label className="col-sm-4 col-form-label">Profile picuture</label>
            <div className="col-sm-8">
              <Avatar
                name={profile?.name}
                url={profile?.profilePicture}
                size="lg"
              />
              <button
                type="button"
                className="btn btn-link"
                style={{ padding: "0" }}
                onClick={onPressUploadProfilePic}
              >
                Upload picture
              </button>
            </div>
          </div>
        </div>
      </div>
 
    

      <button
          type="button"
          className="btn btn-primary"
          style={{ width: "300px" }}
          onClick={onSubmit}
        >
          Update Profile
      </button>

      <input
        type="file"
        className="form-control hidden-input"
        accept="image/*"
        onChange={onSelectProfilePic}
        ref={_profilePicInput}
      />

      
      <ProfilePicCrop
        onRequestClose={onRequestCloseProfilePic}
        profilePicPrev={profilePicPrev}
        userId={profile._id}
        onUpdateCandidate={onUpdateOrg}
        userType={USER_TYPE.ORGANIZATION}
      />
    </div>
  );
};

export default OrganizationProfile;
